@@ -1,8 +1,13 @@
 package Spiffy;
 use strict;
-use warnings;
 use 5.006_001;
-our $VERSION = '0.12';
+use warnings;
+use Carp;
+our $VERSION = '0.13';
+
+my $class_map = {};
+my $options_map = {};
+my $stack_frame = 0; 
 
 sub UNIVERSAL::is_spiffy {
     my $self = shift;
@@ -13,76 +18,11 @@ sub new {
     bless {}, shift;
 }
 
-package DB;
-sub super_args { my @dummy = caller(2); @DB::args }
-package Spiffy;
-
-sub super {
-    @_ = DB::super_args;
-    my $class = ref $_[0] ? ref $_[0] : $_[0];
-    (my $method = (caller(1))[3]) =~ s/.*:://;
-    my $caller_class = caller;
-    my @super_classes = grep {
-        $_ ne $caller_class;
-    } $class->all_my_bases;
-    for my $super_class (@super_classes) {
-        no strict 'refs';
-        next if $super_class eq $class;
-        goto &{"${super_class}::$method"}
-          if $super_class->can($method);
-    }
-    return;
-}
-
-sub attribute {
-    my $package = caller;
-    my ($attribute, $default) = @_;
-    no strict 'refs';
-    return if defined &{"${package}::$attribute"};
-    *{"${package}::$attribute"} =
-        sub {
-            my $self = shift;
-            unless (exists $self->{$attribute}) {
-                $self->{$attribute} = 
-                  ref($default) eq 'ARRAY' ? [] :
-                  ref($default) eq 'HASH' ? {} : 
-                  $default;
-            }
-            return $self->{$attribute} unless @_;
-            $self->{$attribute} = shift;
-        };
-}
-
-my $class_map = {};
-my $options_map = {};
-sub spiffy_constructor_maker {
-    my $spiffy_package = shift;
-    my $caller_package = shift;
-    no strict 'refs';
-    sub {
-        my $name = shift;
-        return if defined &{"${caller_package}::$name"};
-        *{"${caller_package}::$name"} =
-            sub {
-                my $package = caller;
-                my $class = $class_map->{$package}{$name}
-                  or die "No class for ${package}::$name";
-                my $defaults = $options_map->{$package}{$name};
-                $class->new(@$defaults, @_);
-            };
-    }
-}
-
-sub base {
-    push @_, '-base';
-    goto &import;
-}
-
 sub import {
     my $self_package = shift;
     my ($args, @values) = $self_package->parse_arguments(@_);
     my %export_map = map { ($_, 1) } @values;
-    my $caller_package = $args->{package} || caller;
+    my $caller_package = $args->{package} || caller($stack_frame);
     no strict 'refs';
     if ($args->{-base}) {
         push @{"${caller_package}::ISA"}, $self_package;
@@ -111,6 +51,11 @@ sub import {
     }
 }
 
+sub base {
+    push @_, '-base';
+    goto &import;
+}
+
 sub all_my_bases {
     my $class = shift;
     my @bases = ($class);
@@ -122,6 +67,43 @@ sub all_my_bases {
     my @x = grep {not $used->{$_}++} @bases;
 }
 
+sub attribute {
+    my $package = caller;
+    my ($attribute, $default) = @_;
+    no strict 'refs';
+    return if defined &{"${package}::$attribute"};
+    *{"${package}::$attribute"} =
+        sub {
+            my $self = shift;
+            unless (exists $self->{$attribute}) {
+                $self->{$attribute} = 
+                  ref($default) eq 'ARRAY' ? [] :
+                  ref($default) eq 'HASH' ? {} : 
+                  $default;
+            }
+            return $self->{$attribute} unless @_;
+            $self->{$attribute} = shift;
+        };
+}
+
+sub spiffy_constructor_maker {
+    my $spiffy_package = shift;
+    my $caller_package = shift;
+    no strict 'refs';
+    sub {
+        my $name = shift;
+        return if defined &{"${caller_package}::$name"};
+        *{"${caller_package}::$name"} =
+            sub {
+                my $package = caller;
+                my $class = $class_map->{$package}{$name}
+                  or die "No class for ${package}::$name";
+                my $defaults = $options_map->{$package}{$name};
+                $class->new(@$defaults, @_);
+            };
+    }
+}
+
 sub parse_arguments {
     my $class = shift;
     my ($args, @values) = ({}, ());
@@ -147,6 +129,74 @@ sub parse_arguments {
 sub boolean_arguments { () }
 sub paired_arguments { () }
 
+#===============================================================================
+# It's super, man.
+#===============================================================================
+package DB;
+sub super_args { my @dummy = caller(2); @DB::args }
+package Spiffy;
+
+sub super {
+    @_ = DB::super_args;
+    my $class = ref $_[0] ? ref $_[0] : $_[0];
+    (my $method = (caller(1))[3]) =~ s/.*:://;
+    my $caller_class = caller;
+    my @super_classes = grep {
+        $_ ne $caller_class;
+    } $class->all_my_bases;
+    for my $super_class (@super_classes) {
+        no strict 'refs';
+        next if $super_class eq $class;
+        goto &{"${super_class}::$method"}
+          if $super_class->can($method);
+    }
+    return;
+}
+
+#===============================================================================
+# This code deserves a spanking, because it is being very naughty.
+# It is exchanging base.pm's import() for its own, so that people
+# can use base.pm with Spiffy modules, without being the wiser.
+#===============================================================================
+my $real_base_import;
+
+BEGIN {
+    require base unless defined $INC{'base.pm'};
+    $real_base_import = \&base::import;
+    no warnings;
+    *base::import = \&spiffy_base_import;
+}
+
+my $i = 0;
+while (my $caller = caller($i++)) {
+    next unless $caller eq 'base';
+    croak 
+    "Spiffy.pm must be loaded before calling 'use base' with a Spiffy module\n",
+    "See the documentation of Spiffy.pm for details\n  ";
+}
+
+sub spiffy_base_import {
+    my @base_classes = @_;
+    shift @base_classes;
+    no strict 'refs';
+    goto &$real_base_import
+      unless grep {
+          eval "require $_" unless %{"$_\::"};
+          $_->isa('Spiffy');
+      } @base_classes;
+    my $inheritor = caller(0);
+    for my $base_class (@base_classes) {
+        next if $inheritor->isa($base_class);
+        croak "Can't mix Spiffy and non-Spiffy classes in 'use base'.\n", 
+              "See the documentation of Spiffy.pm for details\n  "
+          unless $base_class->isa('Spiffy');
+        $stack_frame = 1; # tell import to use differnt caller
+        import($base_class, $inheritor, '-base');
+        $stack_frame = 0;
+    }
+}
+# END of naughty code.
+
 #===============================================================================
 # Debugging support
 #===============================================================================
@@ -349,6 +399,20 @@ arguments.
         $self->bar(42);
     }
 
+=item * base
+
+This function will call Spiffy::import at runtime, basically making the
+calling package a subclass of whatever object or class base() was
+called on.
+
+    package XXX;
+    BEGIN { require YYY; YYY->base }
+
+is the same as:
+
+    package XXX;
+    use YYY '-base';
+
 =item * XXX
 
 The C<XXX> method will die with a YAML Dump of all the arguments passed
@@ -368,6 +432,56 @@ module being used.
 Any non-paired parameters act like a normal import list; just like those
 used with the Exporter module.
 
+=head1 USING Spiffy WITH base.pm
+
+The proper way to use a Spiffy module as a base class is with the <-base>
+parameter to the C<use> statement. This differs from typical modules where you
+would want to C<use base>.
+
+    package Something;
+    use Spiffy::Module '-base';
+    use base 'NonSpiffy::Module';
+
+Now it may be hard to keep track of what's Spiffy and what is not.
+Therefore Spiffy has actually been made to work with base.pm. You can
+say:
+
+    package Something;
+    use base 'Spiffy::Module';
+    use base 'NonSpiffy::Module';
+
+==head2 base.pm Caveats
+
+To make Spiffy work with base.pm a dirty trick was played. Spiffy swaps
+<base::import> with its own version. If the base modules are not Spiffy,
+Spiffy calls the original base::import. If the base modules are Spiffy,
+then Spiffy does its own thing.
+
+There are two caveats.
+
+=over 4
+
+=item * Spiffy must be loaded first.
+
+If Spiffy is not loaded and C<use base> is invoked on a Spiffy module,
+Spiffy will die with a useful message telling the author to read this
+documentation. That's because Spiffy needed to do the import swap
+beforehand.
+
+If you get this error, simply put a statement like this up front in
+your code:
+
+    use Spiffy ();
+
+=item * No Mixing
+
+C<base.pm> can take multiple arguments. And this works with Spiffy as
+long as all the base classes are Spiffy, or they are all non-Spiffy. If
+they are mixed, Spiffy will die. In this case just use separate C<use
+base> statements.
+
+=back
+
 =head1 AUTHOR
 
 Brian Ingerson <INGY@cpan.org>
